
/*aducprog.c*/
/*-----------------------------------------------------------------------
* Licensed under GNU General Public License
*
*aducprog :  downloads programs from hex file to Analog Devices ADuC family microcontroller
*
*Copyright (C) 2002, 2005 Alex Faveluke
*
*Version 0.3 by  Karl Hiramoto(Karl@hiramoto.org)
*Added -h --Device --R flags
*
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version. This program
*is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*See the GNU General Public License for more details. You should
*have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*license copy supplied as gpl.txt in build directory
*
*            --------Revision History------------
*	v0.3 2005-10-03 adds -h --Device --R flags
*	v0.2 2005-01-03 adds ability to chose ttyS1 instead of ttyS0
* 	last version:  v0.1 2002-07-22
*
*          -----------Usage--------------------
* aducprog [OPTIONS] <inputfile.hex> 
* where options are:   -i  interrogate only
*                        -e  erase internal program memory
*                        -p  program internal program memory
*                        -s  set security modes
*                        -r  jump to user code addr
*                        -t  set ETIM registers
*                        -d  program internal data memory
*                        -z  erase internal prog and data memory
*			-1  use ttyS1 instead of ttyS0
*                       --Device [Device name]
*                        --R  Reset/DLoad  Assume DLoad=RTS line and Reset=DTR line
*                        -h  print help options
*
*-----------------------------------------------------------------------------*/

/*main program file.. includes global definitions, irq handler, interface
and writing loops.*/


#define _GNU_SOURCE             //GNU extensions.. for long options
#include <errno.h>
#include <stdio.h>
#include <string.h>             //for bzero.
#include <stdlib.h>
#include <getopt.h>             //for getopt_long function


#include <unistd.h>             //for read/write
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>            // terminal I/O

                //#include <signal.h>  later...

#include "globdefs.h"
#include "debugfuncs.h"
#include "userint.h"
#include "errhand.h"
#include "pacfuncs.h"           //funcs to deal with packets.
#include "serialdefs.h"         //for serial port definitions

#define ADACK 0x06              //analog devices ACK response
#define ADNACK 0x07

#define ETIM1 0x52              //analog devices ETIM register values.
#define ETIM2 0x04              //May be a problem with the default values... these are
#define ETIM3 0xC9              //what the WSD sets..


void sersetup ();               //sets up serial port for comm
void serrestore ();
void interrogate ();            //interrogates target
void eraseprogrammem ();        //erases program memory
void eraseproganddatamem ();
void writeetim ();              //writes ETIM bits... problem with default???
void progprogmem ();
void checkhexfile ();           //checks a intel hex file
void enable_fast(void);
void runprog (void);
void enable_boot(void);
void disable_boot(void);

int SetDTRLine(int sport_fd,int State);  //toggle high or low
int SetRTSLine(int sport_fd,int State);


Status status;                  //global status struct defined here.

int sport_fd;                   //serial port file descriptor
struct termios oldtio, newtio;  //previous and new serial port settings

FILE *infile;                   //input file pointer
FILE *lfile;                    //"list file" pointer

char infilename[256];
char listfilename[256];


int
main (int argc, char *argv[])
{

  static int retval = 0;        //no errors to start.
  int optc;
  char *timestring;

  int argindex;

  static struct option longopts[] =     //define the darned options array
  {
    {"interrogate", 0, &status.interronly, TRUE},
    {"eraseprogm", 0, &status.eraseprogm, TRUE},
    {"progprogm", 0, &status.progprogm, TRUE},
    {"setsecurity", 0, &status.setsecurity, TRUE},
    {"runataddr", optional_argument, &status.runataddr, TRUE},
    {"tset", optional_argument, &status.etim, TRUE},
    {"dataprog", 0, &status.progdatam, TRUE},
    {"zapall", 0, &status.eraseallm, TRUE},
    {"checkfile", 0, &status.checkfile, TRUE},
    {"Device", 1, 0, 'D'},		//use device name 
    {"RTSDTR", 0, &status.RTSDTR, 'R'}, // Degree C using DTR for reset and RTS for DLoad
    {"help", 0, 0, 'h'},
    {0, 0, 0, 0}                //terminator field
  };

  bzero (&status, sizeof (status));     //wipe status clean at start of program
  bzero (infilename, sizeof (infilename));
  bzero (listfilename, sizeof (listfilename));
  timestring = getdatetime ();

  strncpy (status.starttimest, timestring, 24); //copy start time into status

//  status.RTSDTR=0; // default

  while (1)                     //loop to get all options
    {
      int option_index = 0;     //required for getopt_long.
      optc =
        getopt_long (argc, argv, "iepscr1h::t::dz", longopts, &option_index);
      if (optc == -1)
        {
          argindex = optind;
          break;
        }                       //end of options
      //optind is a pre-defined global..
      switch (optc)
        {
        case 0:
          break;                //long option set a flag
        case 'i':
          status.interronly = TRUE;
          break;
        case 'e':
          status.eraseprogm = TRUE;
          break;
        case 'p':
          status.progprogm = TRUE;
          break;
        case 's':
          status.setsecurity = TRUE;
          break;
        case 'r':
          status.runataddr = TRUE;
          break;
        case 't':
          status.etim = TRUE;
          break;
        case 'd':
          status.progdatam = TRUE;
          break;
        case 'z':
          status.eraseallm = TRUE;
          break;
        case 'c':
          status.checkfile = TRUE;
          break;
        case 'h':  // help for -h option
          printf("aducprog help options\n");
          printf(" -h    Help options this screen \n");
          printf(" aducprog [OPTIONS] <inputfile.hex> \n");
          printf("      -i  interrogate only\n");
          printf("      -e  erase internal program memory\n");
          printf("      -p  program internal program memory\n");
          printf("      -s  set security modes\n");
          printf("      -r  jump to user code addr\n");
          printf("      -t  set ETIM registers\n");
          printf("      -d  program internal data memory\n");
          printf("      -z  erase internal prog and data memory\n");
          printf("      -1  use ttyS1 instead of ttyS0 \n");
          printf("      --Device [device name]\n");
          printf("      --R  Reset/DLoad  Assume DLoad=RTS line and Reset=DTR line\n\n");
	  printf("     Example:  ./aducprog  -i -t -z -p  --Device /dev/tts/USB0  --R /tmp/hexfile.hex \n");


          break;
        case 'D':
          strncpy(status.Device_Name,optarg,62);
          printf ("\nDEBUG.. Serial Dev=%s\n",status.Device_Name);
          status.serdev = 2;
          break;
        case 'R':
          printf ("\nDEBUG.. Reseting device with Dload");
          status.RTSDTR = 1;
          break;
        case '1':
          status.serdev = 1;
          printf ("\nDEBUG.. using ttyS1\n");
          fflush (NULL);
          break;
        }
    }

//get the  input file name

  if (argindex >= argc)
    {
      warn ("no input file");
      bzero (infilename, 256);
    }
  else                          //ought to add a check for >256 length name
    strncpy (infilename, argv[argindex++], 256);
  if (argindex < argc)
    {
      strncpy (listfilename, argv[argindex++], 256);
    }
  else
    bzero (listfilename, 256);
  if (argindex < argc)
    warn ("unused arguments from caller");


#if 0

  if (status.interronly)
    {
      if (status.serialopen != TRUE)
        sersetup ();
      if (status.serialopen < 1)
        die ("serial port open failed before interrogate");
      interrogate ();
      dumpstatus ();
    }

  if (status.etim)
    {
      if (status.serialopen != TRUE)
        sersetup ();
      if (status.serialopen != TRUE)
        die ("serial port open failed before erase all");
      if (status.interrogated != TRUE)
        interrogate ();
      if (status.interrogated != TRUE)
        die ("interrogate failed before erase all");
      writeetim ();
    }
#endif

  if (status.eraseprogm)
    {
	    printf("ERASE\n");
      if (status.serialopen != TRUE)
        sersetup ();
      if (status.serialopen != TRUE)
        die ("serial port open failed before erase program");
      if (status.interrogated != TRUE)
        interrogate ();
      if (status.interrogated != TRUE)
        die ("interrogate failed before erase program");
      eraseprogrammem ();
    }

  if (status.eraseallm)
    {
	    printf("ERASEALL\n");
      if (status.serialopen != TRUE)
        sersetup ();
      
      if (status.serialopen != TRUE)
        die ("serial port open failed before erase all");
      if (status.interrogated != TRUE)
        interrogate ();
//      enable_fast();
      if (status.interrogated != TRUE)
        die ("interrogate failed before erase all");
      eraseproganddatamem ();
    }



  if (status.progprogm)
    {
	    printf("PROGRAM\n");
      if (status.serialopen != TRUE)
        sersetup ();
      if (status.serialopen != TRUE)
        die ("serial port open failed before progprogm");
      if (status.interrogated != TRUE)
        interrogate ();
      if (status.interrogated != TRUE)
        die ("interrogate failed before progprogm");
      
#if 0
      status.interrogated = FALSE;
      if (status.interrogated != TRUE)
        interrogate ();
      if (status.interrogated != TRUE)
        die ("interrogate failed before progprogm");
#endif

      progprogmem ();
      disable_boot();
//      enable_boot();
      runprog();
    } else {
	    printf("RUN\n");
      if (status.serialopen != TRUE)
        sersetup ();
      if (status.serialopen != TRUE)
        die ("serial port open failed before progprogm");
      if (status.interrogated != TRUE)
        interrogate ();
      if (status.interrogated != TRUE)
        die ("interrogate failed before progprogm");
      runprog();
    }

  if (status.checkfile)         //just check the intel hex file..
    {
      checkhexfile ();
    }

  if (status.serialopen == TRUE)
    serrestore ();
  return (retval);              //return from main.  Give back error code if problems
}                               //end main



void
sersetup ()
{
  if (status.serdev == 0)
    {
      sport_fd = open (SERDEV, O_RDWR | O_NOCTTY);
      if (sport_fd < 0)
        {
          perror (SERDEV);
          exit (sport_fd);
        }
    }
  else  if (status.serdev == 1)

    {
      sport_fd = open (SERDEV1, O_RDWR | O_NOCTTY);
      if (sport_fd < 0)
        {
          perror (SERDEV1);
          exit (sport_fd);
        }
    }
  else  if (status.serdev == 2)
    {
      sport_fd = open (status.Device_Name, O_RDWR | O_NOCTTY);
      if (sport_fd < 0)
        {
          perror (SERDEV);
          exit (sport_fd);
        }
    }


#if 0
  // if we need to reset the device in Dload mode
  if (status.RTSDTR)
  {
  // Set both low
    printf("Debug:  reseting device using RTSDTR\n");
    SetDTRLine(sport_fd,0);
    SetRTSLine(sport_fd,0);
    usleep(10000); //delay 10ms for state change
    SetDTRLine(sport_fd,1); // enable
    usleep(100000); //wait 100ms for response to come back
    
  }
#endif

  tcgetattr (sport_fd, &oldtio);        //save existing settings

  bzero (&newtio, sizeof (newtio));     //zero out settings
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
/*baudrate, flow control, 8bit, ignore modem control lines, enable rxvr*/

  newtio.c_iflag = IGNPAR;      //ignore parity and frame errors
  newtio.c_oflag = 0;
  newtio.c_cc[VTIME] = 20;      //if no talky in 2 sec, read returns 0;
  //gotta fix this for nice timeout behavior
  newtio.c_cc[VMIN] = 0;        //blocking read until 1 char recieved
  tcflush (sport_fd, TCIOFLUSH);        //ditches unread or unwrit data
  tcsetattr (sport_fd, TCSANOW, &newtio);       //apply new settings NOW..
  status.serialopen = TRUE;     //should error check everything!
  //way to do a loopback check?
}

void ser_fast_mode(void)
{
  bzero (&newtio, sizeof (newtio));     //zero out settings
  newtio.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
/*baudrate, flow control, 8bit, ignore modem control lines, enable rxvr*/

  newtio.c_iflag = IGNPAR;      //ignore parity and frame errors
  newtio.c_oflag = 0;
  newtio.c_cc[VTIME] = 20;      //if no talky in 2 sec, read returns 0;
  //gotta fix this for nice timeout behavior
  newtio.c_cc[VMIN] = 0;        //blocking read until 1 char recieved
  tcflush (sport_fd, TCIOFLUSH);        //ditches unread or unwrit data
  tcsetattr (sport_fd, TCSANOW, &newtio);       //apply new settings NOW..
}

void
serrestore ()
{
  if (status.serialopen != TRUE)
    {
      warn ("Serial port not open on serrestore call!");
      return;
    }
  tcflush (sport_fd, TCIOFLUSH);
  if (tcsetattr (sport_fd, TCSANOW, &oldtio))
    {
      perror ("tcsetattr");
      warn ("error from tcsetattr in serrestore");

    }
  if (close (sport_fd))
    {
      perror ("close");
      warn ("error from close in serrestore!");
    }

  status.serialopen = FALSE;
  return;
}

void
interrogate ()
{
  char outbuf[24];
  char inbuf[36];
  char packhold[36];
  int charswrit, strpos;


  tcflush (sport_fd, TCIOFLUSH);        //ditches unread or unwrit data
  outbuf[0] = '!';
  outbuf[1] = 'Z';
  outbuf[2] = 0;
  outbuf[3] = 0xA6;

  charswrit = write (sport_fd, outbuf, 4);

  tcdrain (sport_fd);           //equivelant of fflush on serial port!
  if (charswrit != 4)
    warn ("wrote <> 4 chars in interrogate!");

  strpos = 0;

  for (strpos = 0; strpos < 25; strpos++)
    {
      if (read (sport_fd, &inbuf[strpos], 1) < 1)
        die ("read problem in interrogate");
    }



  inbuf[25] = 0;                //terminate in case anyone wants to stick a
  // printf to look at the response.

//Now... strip out the goodies and put in status  ADI has fixed position
//for all values...

  memcpy (packhold, inbuf, 24);

/*if(bchecksum(packhold,24) != inbuf[24]) {
  warn ("CHECKSUM FAIL ON INTERROGATE RESPONSE!");
  status.interrogated = FALSE;
  return;
}
*/

  memcpy (status.product_id, inbuf, 10);
  status.product_id[10] = 0;    //terminate string...
  //although initial bzero should've done
  memcpy (status.firmware_id, (inbuf + (size_t) 10), 4);
  status.firmware_id[4] = 0;

  memcpy (status.hardware_config, (inbuf + (size_t) 16), 2);
  status.hardware_config[2] = 0;
  status.interrogated = TRUE;
}

void
eraseprogrammem ()
{
  char cmdpack[8];
  int charswrit;
  char response;


  cmdpack[0] = 0x07;
  cmdpack[1] = 0x0E;
  cmdpack[2] = 1;
  cmdpack[3] = 'C';
  cmdpack[4] = bchecksum (&cmdpack[2], 2);

  charswrit = write (sport_fd, cmdpack, 5);
  tcdrain (sport_fd);

  if (charswrit != 5)
    warn ("wrote <> 5 chars in eraseprogrammem");
  if (read (sport_fd, &response, 1) < 1)
    die ("couldn't read response in eraseprogrammem");

  switch (response)
    {
    case ADACK:
      printf ("\nerase program command acknowledged by target\n");
      break;
    case ADNACK:
      printf ("\nerase program command NACK'd by target\n");
      break;
    default:
      warn
        ("\nerase program command got other than ACK or NACK from target!\n");
      break;
    }
}                               //end eraseprogrammem()

void
runprog ()                    //fixed ETIM values now... UGLY, works for 11.0592 xtal.
                                //Got the values from the calculation in datasheet, and
                                //copied from what WSD is sending.
                                //Got NACKS on programming if I omit the ETIM write!
                                //with their demo board.
{
  char cmdpack[8];
  int charswrit;
  char response;

#define BOOT 1

  cmdpack[0] = 0x07;
  cmdpack[1] = 0x0E;
  cmdpack[2] = 0x04;
  cmdpack[3] = 'U';
  cmdpack[4] = 0;
#if BOOT
  cmdpack[5] = 0xe0;
#else
  cmdpack[5] = 0;
#endif
  cmdpack[6] = 0;
  cmdpack[7] = bchecksum (&cmdpack[2], 5);

  charswrit = write (sport_fd, cmdpack, 8);
  tcdrain (sport_fd);

  if (charswrit != 8)
    warn ("wrote <> 8 chars in writeetim");
  if (read (sport_fd, &response, 1) < 1)
    die ("couldn't read response in writeetim");

  switch (response)
    {
    case ADACK:
      printf ("\nrun command acknowledged by target\n");
      break;
    case ADNACK:
      printf ("\nrun command NACK'd by target\n");
      break;
    default:
      warn ("\nrun command got other than ACK or NACK from target!\n");
      break;
    }
}                               //end writeetim()

void enable_boot(void)
{
	char cmdpack[8];
	int charswrit;
	char response;

	cmdpack[0] = 0x07;
	cmdpack[1] = 0x0E;
	cmdpack[2] = 0x02;
	cmdpack[3] = 0x46;
	cmdpack[4] = 0xfe;
	cmdpack[5] = 0xba;

	charswrit = write (sport_fd, cmdpack, 6);
	tcdrain (sport_fd);

	if (charswrit != 6)
		warn ("wrote <> 6 chars in writeetim");
	if (read (sport_fd, &response, 1) < 1)
		die ("couldn't read response in writeetim");

	switch (response) {
	case ADACK:
		printf ("\nboot command acknowledged by target\n");
		break;
	case ADNACK:
		printf ("\nboot command NACK'd by target\n");
		break;
	default:
		warn ("\nboot command got other than ACK or NACK from target!\n");
		break;
	}
}

void disable_boot(void)
{
	char cmdpack[8];
	int charswrit;
	char response;

	cmdpack[0] = 0x07;
	cmdpack[1] = 0x0E;
	cmdpack[2] = 0x02;
	cmdpack[3] = 0x46;
	cmdpack[4] = 0xff;
	cmdpack[5] = 0xb9;

	charswrit = write (sport_fd, cmdpack, 6);
	tcdrain (sport_fd);

	if (charswrit != 6)
		warn ("wrote <> 6 chars in writeetim");
	if (read (sport_fd, &response, 1) < 1)
		die ("couldn't read response in writeetim");

	switch (response) {
	case ADACK:
		printf ("\nboot command acknowledged by target\n");
		break;
	case ADNACK:
		printf ("\nboot command NACK'd by target\n");
		break;
	default:
		warn ("\nboot command got other than ACK or NACK from target!\n");
		break;
	}
}

void
writeetim ()                    //fixed ETIM values now... UGLY, works for 11.0592 xtal.
                                //Got the values from the calculation in datasheet, and
                                //copied from what WSD is sending.
                                //Got NACKS on programming if I omit the ETIM write!
                                //with their demo board.
{
  char cmdpack[8];
  int charswrit;
  char response;


  cmdpack[0] = 0x07;
  cmdpack[1] = 0x0E;
  cmdpack[2] = 0x04;
  cmdpack[3] = 'T';
  cmdpack[4] = ETIM1;
  cmdpack[5] = ETIM2;
  cmdpack[6] = ETIM3;
  cmdpack[7] = bchecksum (&cmdpack[2], 5);

  charswrit = write (sport_fd, cmdpack, 8);
  tcdrain (sport_fd);

  if (charswrit != 8)
    warn ("wrote <> 8 chars in writeetim");
  if (read (sport_fd, &response, 1) < 1)
    die ("couldn't read response in writeetim");

  switch (response)
    {
    case ADACK:
      printf ("\nwrite etim command acknowledged by target\n");
      break;
    case ADNACK:
      printf ("\nwrite etim command NACK'd by target\n");
      break;
    default:
      warn ("\nwrite etim command got other than ACK or NACK from target!\n");
      break;
    }
}                               //end writeetim()

void enable_fast(void)
{
  char cmdpack[8];
  int charswrit;
  char response;


  cmdpack[0] = 0x07;
  cmdpack[1] = 0x0E;
  cmdpack[2] = 0x03;
  cmdpack[3] = 'B';
  cmdpack[4] = 0x86; /* 115200bps on 11M crystal */
  cmdpack[5] = 0x08;
  cmdpack[6] = bchecksum (&cmdpack[2], 4);

  charswrit = write (sport_fd, cmdpack, 7);
  tcdrain (sport_fd);

  if (charswrit != 7)
    warn ("wrote <> 7 chars in writeetim");
 

  if (read (sport_fd, &response, 1) < 1)
    die ("couldn't read response in writeetim");

  switch (response)
    {
    case ADACK:
      printf ("\nFast mode enabled, ACK\n");
      break;
    case ADNACK:
      printf ("\nFast mode NACK\n");
      break;
    default:
      warn ("\nFast Mode: command got other than ACK or NACK from target!\n");
      break;
    }
  
//	ser_fast_mode();
	usleep(1000);
}                               //end writeetim()

void
eraseproganddatamem ()
{
  char cmdpack[8];
  int charswrit;
  char response;

  cmdpack[0] = 0x07;
  cmdpack[1] = 0x0E;
  cmdpack[2] = 1;
  cmdpack[3] = 'A';
  cmdpack[4] = bchecksum (&cmdpack[2], 2);

  charswrit = write (sport_fd, cmdpack, 5);
  tcdrain (sport_fd);

  if (charswrit != 5)
    warn ("wrote <> 5 chars in eraseproganddatamem");
  if (read (sport_fd, &response, 1) < 1)
    die ("couldn't read response in eraseproganddatamem");

  switch (response)
    {
    case ADACK:
      printf ("\nerase program and data command acknowledged by target\n");
      break;
    case ADNACK:
      printf ("\nerase program and data command NACK'd by target\n");
      break;
    default:
      warn
        ("\nerase program and data command got other than ACK or NACK from target!\n");
      break;
    }
}                               //end eraseproganddatamem()

void
progprogmem ()
{
  char inlinebuf[256];
  char datapack[36];
  char progpack[36];
  char response;
  int progpacklen;
  int charswrit;
  int linecount;
  int pageaddr;

  int packtype;
  int byteqty;
  int loadaddr;





  linecount = 0;
  pageaddr = 0;                 //assume always zero for now...



  if (infilename[0] == 0)
    {
      warn ("No input file for programming!  Not doing anything");
      return;
    }
  infile = fopen (infilename, "r");
  if (infile == NULL)
    {
      warn ("trouble opening input file!  Not programming");
      perror ("fopen in progprogm");
      return;
    }

//Extra whitespace in the hex file would hose this.

  while ((fscanf (infile, "%255s", inlinebuf)) != EOF)
    {
      linecount++;
      //printf("\ngot line #%i: %s",linecount, inlinebuf);
      if (intelhex2data (&packtype, &byteqty, &loadaddr, datapack, inlinebuf)
          != 0)
        {
          warn ("error returned from intelhex2data!!");
          return;               //don't program bad data!
        }

      if (packtype == 0)
        {
          progpacklen =
            data2ADprogpack (progpack, datapack, pageaddr, loadaddr, byteqty,
                             FALSE);

          charswrit = write (sport_fd, progpack, progpacklen);
          tcdrain (sport_fd);

          if (charswrit != progpacklen)
            die ("didn't write all packet in progprogm");


          if (read (sport_fd, &response, 1) < 1)
            die ("couldn't read response in progprogm");

          switch (response)
            {
            case ADACK:
              printf ("\nprogram hexfile line %i ACK'd by target", linecount);
              break;
            case ADNACK:
              printf ("\nprogram hexfile line %i NACK'd by target\n", linecount);       //try again?
              die ("bailing out on NACK in program routine!");
              break;
            default:
              warn ("\nprogram got other than ACK or NACK from target!\n");
              die ("bailing out on neither ACK or NACK in program routine!");
              break;
            }
        }

      else if (packtype == 1)
        {
          break;
        }                       // bust out of while loop on end packet.
      else
        warn ("Got packet type other than 0 or 1 in progprogm!");
    }

  printf ("\nprogramming completed...\n");
}


void
checkhexfile ()
{
  char inlinebuf[256];
  char datapack[36];


  int linecount;
  int pageaddr;
  int result;

  int packtype;
  int byteqty;
  int loadaddr;

  linecount = 0;
  pageaddr = 0;                 //assume always zero for now...



  if (infilename[0] == 0)
    {
      warn ("No input file for checkhexfile  Not doing anything");
      return;
    }
  infile = fopen (infilename, "r");
  if (infile == NULL)
    {
      warn ("trouble opening input file!  Not programming");
      perror ("fopen in progprogm");
      return;
    }

//Extra whitespace in the hex file would hose this.

  while ((fscanf (infile, "%255s", inlinebuf)) != EOF)
    {
      linecount++;
      printf ("\ngot line #%i: %s", linecount, inlinebuf);
      result =
        intelhex2data (&packtype, &byteqty, &loadaddr, datapack, inlinebuf);
      if (result != 0)
        {
          warn ("error returned from intelhex2data!!");
          printf ("\nreturned %i from line #%i", result, linecount);
        }
      printf ("\n---------------\n");
      printf ("read line #%i", linecount);
      printf ("\ntype is %i,  byteqty is %i, load addr is 0x%04x\n", packtype,
              byteqty, loadaddr);
      {
        int i;
        for (i = 0; i < byteqty; i++)
          {
            if (i > 0)
              printf (" ,");
            printf ("0x%02x", (unsigned char) datapack[i]);
          }
      }
      printf ("\n---------------\n\n");

    }

  printf ("\nend of checkhexfile");
}


//change line state
int SetDTRLine(int sport_fd,int State)
{
    int status; // current status
    
      if (ioctl (sport_fd, TIOCMGET, &status) == -1)
      {
            perror("TIOCMGET failed ");
            return -1;        
      }
      
      //on my system
      //TIOCMGET and the like are defined in /usr/include/asm/ioctls.h
      //which is included from /usr/include/bits/ioctls.h
      //which is included from /usr/include/sys/ioctl.h
      
      
      
    if (State)  // set bit
    {
        status |= TIOCM_DTR;
         
        if (ioctl (sport_fd, TIOCMSET, &status) == -1)
        {
            perror("TIOCMSET failed ");
            return -1;        
        }

    }  
    else // clear bit
    {
        status &= ~TIOCM_DTR;
        if (ioctl (sport_fd, TIOCMSET, &status) == -1)
        {
            perror("TIOCMSET failed ");
            return -1;        
        }
        

    
    }
          
      
    return 0;
}

int SetRTSLine(int sport_fd,int State)
{
    int status; // current status
    
      if (ioctl (sport_fd, TIOCMGET, &status) == -1)
      {
            perror("TIOCMGET failed ");
            return -1;        
      }
      
      
    if (State)  // set bit
    {
        status |= TIOCM_RTS;
         
        if (ioctl (sport_fd, TIOCMSET, &status) == -1)
        {
            perror("TIOCMSET failed ");
            return -1;        
        }

    }  
    else // clear bit
    {
        status &= ~TIOCM_RTS;
        if (ioctl (sport_fd, TIOCMSET, &status) == -1)
        {
            perror("TIOCMSET failed ");
            return -1;        
        }
        

    
    }
          
      
    return 0;
}

